// Lightray Shader (inspired by shader work done by Evolved)

float4 SkyColor : Diffuse
<    string UIName =  "SkyColor";    
> = {1.0, 1.0, 1.0, 1.0f};

//--------------
// un-tweaks
//--------------
   matrix ViewProjection:ViewProjection;
   matrix ProjMat={0.5,0,0,0.5,0,-0.5,0,0.5,0,0,0.5,0.5,0,0,0,1}; 

//--------------
// tweaks
//--------------
   float3 LightDir={-0.0,-0.5,-0.723};
   float Density=1.0f;
   float AlphaAngle=1.0;
   float LightRayLength=0.5;
   float LightRayQuality=15.0f;
   float LightRayDecay=0.975f;
   float LightRayFactor=1.0f;

//--------------
// Textures
//--------------
   texture BlackTextureTX <string Name=" ";>;
   sampler BlackTexture=sampler_state 
      {
	Texture=<BlackTextureTX>;
   	ADDRESSU = CLAMP;
   	ADDRESSV = CLAMP;
   	ADDRESSW = CLAMP;
      };

//--------------
// structs 
//--------------
   struct InPut
     {
 	float4 Pos:POSITION;
 	float2 UV:TEXCOORD; 
     };
   struct OutPut
     {
	float4 OPos:POSITION; 
 	float2 Tex:TEXCOORD0;
     };

//--------------
// vertex shader
//--------------
   OutPut VS(InPut IN) 
     {
 	OutPut OUT;
	OUT.OPos=IN.Pos; 
 	OUT.Tex=IN.UV;
	return OUT;
    }

// pixel shader (any way to pass in SampleNum and still unroll)

float4 PS10(OutPut IN) : COLOR
{
	int SampleNum = 10;
	float4 ScreenToLight=mul(ProjMat,mul(-LightDir,ViewProjection));
	float2 DeltaTex=((ScreenToLight.xy/ScreenToLight.z)-IN.Tex)*sign(ScreenToLight.z);
	float thelength=length(DeltaTex);DeltaTex /=thelength;
	thelength=min(thelength,Density);DeltaTex *=thelength;
	float2 NewUv=IN.Tex;
	float3 Scatter=0;
	float FallOff=1.0;
	DeltaTex /= SampleNum;
	DeltaTex *= LightRayLength;
	for (int i=0; i < SampleNum; i++)
	 {		
	  NewUv += (DeltaTex*LightRayLength);
	  Scatter += (tex2D(BlackTexture,NewUv)*AlphaAngle)*FallOff;
	  FallOff *= LightRayDecay;
	 }
	Scatter /= SampleNum;
	return float4(Scatter*SkyColor.xyz*SkyColor.a*LightRayFactor,1);
}

float4 PS20(OutPut IN) : COLOR
{
	int SampleNum = 20;
	float4 ScreenToLight=mul(ProjMat,mul(-LightDir,ViewProjection));
	float2 DeltaTex=((ScreenToLight.xy/ScreenToLight.z)-IN.Tex)*sign(ScreenToLight.z);
	float thelength=length(DeltaTex);DeltaTex /=thelength;
	thelength=min(thelength,Density);DeltaTex *=thelength;
	float2 NewUv=IN.Tex;
	float3 Scatter=0;
	float FallOff=1.0;
	DeltaTex /= SampleNum;
	DeltaTex *= LightRayLength;
	for (int i=0; i < SampleNum; i++)
	 {		
	  NewUv += (DeltaTex*LightRayLength);
	  Scatter += (tex2D(BlackTexture,NewUv)*AlphaAngle)*FallOff;
	  FallOff *= LightRayDecay;
	 }
	Scatter /= SampleNum;
	return float4(Scatter*SkyColor.xyz*SkyColor.a*LightRayFactor,1);
}

float4 PS35(OutPut IN) : COLOR
{
	int SampleNum = 35;
	float4 ScreenToLight=mul(ProjMat,mul(-LightDir,ViewProjection));
	float2 DeltaTex=((ScreenToLight.xy/ScreenToLight.z)-IN.Tex)*sign(ScreenToLight.z);
	float thelength=length(DeltaTex);DeltaTex /=thelength;
	thelength=min(thelength,Density);DeltaTex *=thelength;
	float2 NewUv=IN.Tex;
	float3 Scatter=0;
	float FallOff=1.0;
	DeltaTex /= SampleNum;
	DeltaTex *= LightRayLength;
	for (int i=0; i < SampleNum; i++)
	 {		
	  NewUv += (DeltaTex*LightRayLength);
	  Scatter += (tex2D(BlackTexture,NewUv)*AlphaAngle)*FallOff;
	  FallOff *= LightRayDecay;
	 }
	Scatter /= SampleNum;
	return float4(Scatter*SkyColor.xyz*SkyColor.a*LightRayFactor,1);
}

float4 PS80(OutPut IN) : COLOR
{
	int SampleNum = 80;
	float4 ScreenToLight=mul(ProjMat,mul(-LightDir,ViewProjection));
	float2 DeltaTex=((ScreenToLight.xy/ScreenToLight.z)-IN.Tex)*sign(ScreenToLight.z);
	float thelength=length(DeltaTex);DeltaTex /=thelength;
	thelength=min(thelength,Density);DeltaTex *=thelength;
	float2 NewUv=IN.Tex;
	float3 Scatter=0;
	float FallOff=1.0;
	DeltaTex /= SampleNum;
	DeltaTex *= LightRayLength;
	for (int i=0; i < SampleNum; i++)
	 {		
	  NewUv += (DeltaTex*LightRayLength);
	  Scatter += (tex2D(BlackTexture,NewUv)*AlphaAngle)*FallOff;
	  FallOff *= LightRayDecay;
	 }
	Scatter /= SampleNum;
	return float4(Scatter*SkyColor.xyz*SkyColor.a*LightRayFactor,1);
}

float4 PSNone(OutPut IN) : COLOR
{
	return tex2D(BlackTexture,IN.Tex);
}

// techniques   

technique ScatterLOW
{
	pass p1
	{		
		VertexShader = compile vs_3_0 VS(); 
		PixelShader  = compile ps_3_0 PS10();
	}
}

technique ScatterMEDIUM
{
	pass p1
	{		
		VertexShader = compile vs_3_0 VS(); 
		PixelShader  = compile ps_3_0 PS20();
	}
}

technique ScatterHIGH
{
	pass p1
	{		
		VertexShader = compile vs_3_0 VS(); 
		PixelShader  = compile ps_3_0 PS35();
	}
}

technique ScatterHIGHEST
{
	pass p1
	{		
		VertexShader = compile vs_3_0 VS(); 
		PixelShader  = compile ps_3_0 PS80();
	}
}

technique NoScatter
{
	pass p1
	{		
		VertexShader = compile vs_3_0 VS(); 
		PixelShader  = compile ps_3_0 PSNone();
	}
}
