string Description = "Effect Wave Scroll Shader";
#include "settings.fx"

float4x4 WorldViewProjection : WorldViewProjection;
float4x4 World : World;
float4x4 View : View;
float4x4 Projection : Projection;
float4x4 WorldIT : WorldInverseTranspose;
float time : time;
float4 eyePos : CameraPosition;

// PE: better poisson disk spreading.
const uint poissonDisknum = 16;
static const float2 poissonDisk[16] =
   {
      float2(0.2770745f, 0.6951455f),
      float2(0.1874257f, -0.02561589f),
      float2(-0.3381929f, 0.8713168f),
      float2(0.5867746f, 0.1087471f),
      float2(-0.3078699f, 0.188545f),
      float2(0.7993396f, 0.4595091f),
      float2(-0.09242552f, 0.5260149f),
      float2(0.3657553f, -0.5329605f),
      float2(-0.3829718f, -0.2476171f),
      float2(-0.01085108f, -0.6966301f),
      float2(0.8404155f, -0.3543923f),
      float2(-0.5186161f, -0.7624033f),
      float2(-0.8135794f, 0.2328489f),
      float2(-0.784665f, -0.2434929f),
      float2(0.9920505f, 0.0855163f),
      float2(-0.687256f, 0.6711345f)
   };
   
// shadow mapping
matrix          m_mShadow;
float4          m_vCascadeOffset[8];
float4          m_vCascadeScale[8];
int             m_nCascadeLevels;
float           m_fMinBorderPadding;     
float           m_fMaxBorderPadding;
float           m_fShadowBiasFromGUI;  // A shadow map offset to deal with self shadow artifacts.  
float           m_fCascadeBlendArea; // Amount to overlap when blending between cascades.
float           m_fTexelSize; 
float           m_fCascadeFrustumsEyeSpaceDepths[8];
float3          m_vLightDir;

float directionX
<
	string UIWidget = "slider";
	float UIMax = 3.0;
	float UIMin = -3.0;
	float UIStep = 0.01;
> = 0.10;

float directionY
<
	string UIWidget = "slider";
	float UIMax = 3.0;
	float UIMin = -3.0;
	float UIStep = 0.01;
> = 0.00;

float4 clipPlane : ClipPlane;  //cliplane for water plane

float4 LightSource
<   string UIType = "Fixed Light Direction";
> = {-1.0f, -1.0f, -1.0f, 1.0f};

float4 SpotFlashPos;
float4 SpotFlashColor;

float GlobalSurfaceIntensity
<    string UIName =  "GlobalSurfaceIntensity";    
> = {1.0f};

float ShadowStrength
<    string UIName =  "ShadowStrength";    
> = {1.0f};

float SurfaceSunFactor
<    string UIName =  "SurfaceSunFactor";    
> = {1.0f};

float4 FogColor : Diffuse
<   string UIName =  "Fog Color";    
> = {0.0f, 0.0f, 0.0f, 0.0000001f};

float4 HudFogColor : Diffuse
<   string UIName =  "Hud Fog Color";    
> = {0.0f, 0.0f, 0.0f, 0.0000001f};

float4 HudFogDist : Diffuse
<   string UIName =  "Hud Fog Dist";    
> = {1.0f, 0.0f, 0.0f, 0.0000001f};

float4 AmbiColorOverride
<    string UIName =  "AmbiColorOverride";    
> = {1.0f, 1.0f, 1.0f, 1.0f};

float4 AmbiColor : Ambient
<    string UIName =  "AmbiColor";    
> = {0.1f, 0.1f, 0.1f, 1.0f};

float4 SurfColor : Diffuse
<    string UIName =  "SurfColor";    
> = {1.0f, 1.0f, 1.0f, 1.0f};

float4 SkyColor : Diffuse
<    string UIName =  "SkyColor";    
> = {1.0, 1.0, 1.0, 1.0f};

float4 FloorColor : Diffuse
<    string UIName =  "FloorColor";    
> = {1.0, 1.0, 1.00, 1.0f};

float alphaoverride  : alphaoverride;

float4 EntityEffectControl
<    string UIName =  "EntityEffectControl";    
> = {0.0f, 0.0f, 0.0f, 0.0f};

texture DiffuseMap : RENDERCOLORTARGET
<
    string Name = "D.tga";
    string type = "2D";
>;

texture DepthMapTX1 : DiffuseMap
<
    string Name = "DEPTH1.tga";
    string type = "2D";
>;
texture DepthMapTX2 : DiffuseMap
<
    string Name = "DEPTH1.tga";
    string type = "2D";
>;
texture DepthMapTX3 : DiffuseMap
<
    string Name = "DEPTH1.tga";
    string type = "2D";
>;
texture DepthMapTX4 : DiffuseMap
<
    string Name = "D.tga";
    string type = "2D";
>;


sampler2D DiffuseSampler = sampler_state
{
    Texture   = <DiffuseMap>;
    MipFilter = LINEAR;
    MinFilter = ANISOTROPIC;
    MagFilter = LINEAR;
};

sampler2D DepthMap1 = sampler_state
{
   Texture = <DepthMapTX1>;   
#ifdef LINEARSHADOW
   Filter = MIN_MAG_MIP_LINEAR;
   AddressU = MIRROR;
   AddressV = MIRROR;
   //ComparisonFunc = LESS_EQUAL; // ps_4 only
#else
    MinFilter = POINT;//LINEAR;
    MagFilter = POINT;//LINEAR;
    AddressU = Clamp;
    AddressV = Clamp;
#endif
};
sampler2D DepthMap2 = sampler_state
{
   Texture = <DepthMapTX2>;   
#ifdef LINEARSHADOW
   Filter = MIN_MAG_MIP_LINEAR;
   AddressU = MIRROR;
   AddressV = MIRROR;
#else
    MinFilter = POINT;//LINEAR;
    MagFilter = POINT;//LINEAR;
    AddressU = Clamp;
    AddressV = Clamp;
#endif
};
sampler2D DepthMap3 = sampler_state
{
   Texture = <DepthMapTX3>;   
#ifdef LINEARSHADOW
   Filter = MIN_MAG_MIP_LINEAR;
   AddressU = MIRROR;
   AddressV = MIRROR;
#else
    MinFilter = POINT;//LINEAR;
    MagFilter = POINT;//LINEAR;
    AddressU = Clamp;
    AddressV = Clamp;
#endif
};
sampler2D DepthMap4 = sampler_state
{
   Texture = <DepthMapTX4>;   
#ifdef LINEARSHADOW
   Filter = MIN_MAG_MIP_LINEAR;
   AddressU = MIRROR;
   AddressV = MIRROR;
#else
    MinFilter = POINT;//LINEAR;
    MagFilter = POINT;//LINEAR;
    AddressU = Wrap;
    AddressV = Wrap;
    AddressU = Border;
    AddressV = Border;
   BorderColor = 0xFFFFFFFF;
#endif
};


struct app_in
{
	float4 pos : POSITION;
	float4 Normal : NORMAL;
	float2 uv : TEXCOORD0;
};

struct vs_out
{
	float4 pos : POSITION;
	float2 uv : TEXCOORD0;
	float2 uv2 : TEXCOORD1;
    float4 WPos : TEXCOORD2;
	float clip : TEXCOORD3;
	float vDepth : TEXCOORD4;
    float3 LightVec : TEXCOORD5;
    float3 WorldNormal : TEXCOORD6;
    float2 vegshadowuv : TEXCOORD7;	
};

/****** helper functions for shadow mapping*****/

void ComputeCoordinatesTransform( in int iCascadeIndex,
                                  in float4 InterpolatedPosition,
                                  in out float4 vShadowTexCoord ) 
{
   float4 vLightDot = mul ( InterpolatedPosition, m_mShadow );
   vLightDot *= m_vCascadeScale[iCascadeIndex];
   vLightDot += m_vCascadeOffset[iCascadeIndex];
   vShadowTexCoord.xyz = vLightDot.xyz;
} 
       
float texture2DCompare(sampler2D depths, float2 uv, float compare)
{
    float depth = tex2D(depths, uv).r;
    return step(compare, depth);
}

float PCF5x5(sampler2D depths, float2 size, float2 uv, float compare)
{
    float result = 0.0;
    for(int x=-2; x<=2; x++){
        for(int y=-2; y<=2; y++){
            float2 off = float2(x,y)/size;
            result += texture2DCompare(depths, uv+off, compare);
        }
    }
    return result/25.0;
}

float PCF4x4(sampler2D depths, float2 size, float2 uv, float compare)
{
    float result = 0.0;
    float2 texsize = 1.0/size;
    
    for(float x=-1.5; x<=1.5; x+= 1.0){
        for(float y=-1.5; y<=1.5; y+=1.0){
            float2 off = float2(x,y) * texsize;
            result += texture2DCompare(depths, uv+off, compare);
        }
    }
    return result/16.0;
}

float PCF4x4rand(sampler2D depths, float2 size, float2 uv, float compare, float2 mwpos, float pradius)
{
    float result = 0.0;
    float2 texsize = 1.0/size;
    int runs = 0;
    for(float x=-1.5; x<=1.5; x+= 1.0){
        for(float y=-1.5; y<=1.5; y+=1.0){
            //float2 o = 1.0-( fmod(floor(mwpos*(y*x)), 100.0) * 0.01 );
            //int pidx = clamp( floor(fmod(floor(mwpos*(y+x)), poissonDisknum)) , 0 , poissonDisknum );
            //mwpos += float2(x+y,y+x);
            //float2 o = poissonDisk[ clamp(runs,0,poissonDisknum) ];
            //float2 off = (o+float2(x,y)) * texsize;
            float2 off = ( poissonDisk[ clamp(runs,0,poissonDisknum) ] * pradius ) * texsize;

            result += texture2DCompare(depths, uv+off, compare);
            runs++;
        }
    }
    result += texture2DCompare(depths, uv, compare);
    result += texture2DCompare(depths, uv+( float2(0.55,0.55) * texsize ), compare);

    return result/18.0;
}

float PCF3x3rand(sampler2D depths, float2 size, float2 uv, float compare, float2 mwpos, float pradius)
{
    float result = 0.0;
    float2 texsize = 1.0/size;
    int runs = 0;
    for(float x=-1.0; x<=1.0; x+= 1.0){
        for(float y=-1.0; y<=1.0; y+=1.0){
            //float2 o = 1.0-( fmod(floor(mwpos*(y*x)), 100.0) * 0.01 );
            //int pidx = clamp( floor(fmod(floor(mwpos*(y+x)), poissonDisknum)) , 0 , poissonDisknum );
            //mwpos += float2(x+y,y+x);
            //float2 o = poissonDisk[ clamp(runs,0,poissonDisknum) ];
            //float2 off = (o+float2(x,y)) * texsize;
            float2 off = ( poissonDisk[ clamp(runs,0,poissonDisknum) ] * pradius ) * texsize;

            result += texture2DCompare(depths, uv+off, compare);
            runs++;
        }
    }
    return result/9.0;
}

float PCF5x3(sampler2D depths, float2 size, float2 uv, float compare)
{
    float result = 0.0;
    for(int x=-1; x<=1; x++){
        for(int y=-2; y<=2; y++){
            float2 off = float2(x,y)/size;
            result += texture2DCompare(depths, uv+off, compare);
        }
    }
    return result/15.0;
}
float PCF3x3(sampler2D depths, float2 size, float2 uv, float compare)
{
    float result = 0.0;
    for(int x=-1; x<=1; x++){
        for(int y=-1; y<=1; y++){
            float2 off = float2(x,y)/size;
            result += texture2DCompare(depths, uv+off, compare);
        }
    }
    return result/9.0;
}

void CalculatePCFPercentLit ( in int iCurrentCascadeIndex,
                       in float4 vShadowTexCoord, 
                       in float fCurrentDepth,
                       in float2 mwpos,                       
                              out float fPercentLit ) 
{
   fPercentLit = 0.0f;
   float2 fTS = float2(1.0f/SHADOWTEXELX,1.0f/SHADOWTEXELY);
   float2 tsize = float2( SHADOWTEXELX ,SHADOWTEXELY );
   if ( iCurrentCascadeIndex==0 )
   {
        float2 o = ( fmod(floor(mwpos), 10.0) * 0.1 )*fTS;
        fPercentLit = 1.0-PCF4x4rand(DepthMap1, tsize, vShadowTexCoord.xy, vShadowTexCoord.z-0.00003f,mwpos , 2.35 );
   }
   else
   {
      if ( iCurrentCascadeIndex==1 )
      {
           float2 o = (fmod(floor(mwpos), 1.50) * 0.25)*fTS;
           fPercentLit = 1.0-PCF4x4rand(DepthMap2, float2( SHADOWTEXELX * 0.60 ,SHADOWTEXELY * 3.0  )  , vShadowTexCoord.xy , vShadowTexCoord.z-0.00004f,mwpos , 1.65 );
           //fPercentLit = vShadowTexCoord.z-0.00003f > tex2D(DepthMap2,float2(vShadowTexCoord.x,vShadowTexCoord.y) ).x ? 1.0f : 0.0f;
           fPercentLit *= 0.90f;
      }
      else
      {
         if ( iCurrentCascadeIndex==2 )
         {
            fPercentLit += vShadowTexCoord.z-0.000044f > tex2D(DepthMap3,float2(vShadowTexCoord.x,vShadowTexCoord.y+0.000010f  )).x ? 1.0f : 0.0f;
            fPercentLit += vShadowTexCoord.z-0.000044f > tex2D(DepthMap3,float2(vShadowTexCoord.x,vShadowTexCoord.y+0.000010f  )+( (float2( 0.55, 0.0) ) *fTS)).x ? 1.0f : 0.0f;
            fPercentLit *= 0.30f;
         }
         else
         {
            if ( iCurrentCascadeIndex==3 && vShadowTexCoord.z<1.0 )
            {
               fPercentLit += vShadowTexCoord.z-0.00016f > tex2D(DepthMap4,float2(vShadowTexCoord.x,vShadowTexCoord.y+0.00020f )).x ? 0.40f : 0.0f;
            }
         }
      }
   }                                                                             
   float fFadeOutLastCascade = saturate(1.0-((fCurrentDepth- (m_fCascadeFrustumsEyeSpaceDepths[2]+2800) )/ 600 ));
   fPercentLit *= fFadeOutLastCascade;
}

void CalculateBlendAmountForInterval ( in int iCurrentCascadeIndex, 
                                       in out float fPixelDepth, 
                                       in out float fCurrentPixelsBlendBandLocation,
                                       out float fBlendBetweenCascadesAmount ) 
{
    float fBlendInterval = m_fCascadeFrustumsEyeSpaceDepths[ iCurrentCascadeIndex ];
    int fBlendIntervalbelowIndex = min(0, iCurrentCascadeIndex-1);
    if ( fBlendIntervalbelowIndex>1 )
    {
      fPixelDepth -= m_fCascadeFrustumsEyeSpaceDepths[ fBlendIntervalbelowIndex ];
      fBlendInterval -= m_fCascadeFrustumsEyeSpaceDepths[ fBlendIntervalbelowIndex ];
    }
    fCurrentPixelsBlendBandLocation = 1.0f - (fPixelDepth / fBlendInterval);
    fBlendBetweenCascadesAmount = fCurrentPixelsBlendBandLocation / m_fCascadeBlendArea;
}

vs_out DefaultVertexShader( app_in IN )
{
	vs_out OUT;
	float4 tempPos = mul( IN.pos, World );
	float radian = tempPos.x+tempPos.z+(time*2.0f);
	float rise = 1.5f;
	tempPos.y = tempPos.y + rise + cos(radian)*rise;
	OUT.pos = mul( tempPos, View );
	OUT.pos = mul( OUT.pos, Projection );
	OUT.uv = IN.uv + float2(directionX,directionY)*time;
	OUT.uv2 = IN.uv + float2(directionX,directionY)*time*0.7f;
	OUT.WPos = tempPos; 
    OUT.clip = dot(tempPos, clipPlane);                                                                      
    OUT.vDepth = mul( IN.pos, WorldViewProjection ).z; 
    OUT.WorldNormal = normalize(mul(IN.Normal, WorldIT).xyz);
    OUT.LightVec = normalize(LightSource);
    OUT.vegshadowuv = float2(tempPos.x/51200.0f,tempPos.z/51200.0f);
	return OUT;
}

float4 DefaultPixelShader( vs_out IN ) : COLOR
{
    clip(IN.clip);
	float4 diff = tex2D(DiffuseSampler,IN.uv)*0.5f;
	float4 diff2 = tex2D(DiffuseSampler,float2(1-IN.uv2.x,IN.uv2.y))*0.5f;
    float4 diffusemap = diff+diff2;
    float3 Ln = normalize(IN.LightVec);
    float3 V  = (eyePos - IN.WPos);  
    float3 Vn  = normalize(V); 
    float3 Hn = normalize(Vn+Ln);
    float4 lighting = lit(dot(Ln,IN.WorldNormal),dot(Hn,IN.WorldNormal),24);
    lighting.y = lerp(0.65,lighting.y,SurfaceSunFactor);
	
    float fShadow = 0.0f;
    float fCurrentPixelDepth;
    fCurrentPixelDepth = IN.vDepth.x;
    int iCurrentCascadeIndex;
    iCurrentCascadeIndex = 3;
    if ( fCurrentPixelDepth < m_fCascadeFrustumsEyeSpaceDepths[2] ) iCurrentCascadeIndex = 2;
    if ( fCurrentPixelDepth < m_fCascadeFrustumsEyeSpaceDepths[1] ) iCurrentCascadeIndex = 1;
    if ( fCurrentPixelDepth < m_fCascadeFrustumsEyeSpaceDepths[0] ) iCurrentCascadeIndex = 0;
    float4 vShadowMapTextureCoord = 0.0f;
    if ( ShadowStrength > 0.0f )
    {
      float fPercentLit = 0.0f;
      int iNextCascadeIndex = 1;
      iNextCascadeIndex = min ( 4 - 1, iCurrentCascadeIndex + 1 ); 
      float fBlendBetweenCascadesAmount = 1.0f;
      float fCurrentPixelsBlendBandLocation = 1.0f;
      CalculateBlendAmountForInterval ( iCurrentCascadeIndex, fCurrentPixelDepth, fCurrentPixelsBlendBandLocation, fBlendBetweenCascadesAmount );
      float4 finalwpos = IN.WPos;
      ComputeCoordinatesTransform( iCurrentCascadeIndex, finalwpos, vShadowMapTextureCoord );    
      float sbias = lerp(ADJUSTSHADOW,0.000115f,clamp( dot(IN.WorldNormal,Ln) ,0.0,1.0));
      vShadowMapTextureCoord.z -= sbias + (fCurrentPixelDepth/20000000.0f);
      CalculatePCFPercentLit ( iCurrentCascadeIndex, vShadowMapTextureCoord, fCurrentPixelDepth, finalwpos.xz, fShadow );
      if( fCurrentPixelsBlendBandLocation < (m_fCascadeBlendArea) ) 
      {  
         float4 vShadowMapTextureCoord_blend = 0.0f;
         ComputeCoordinatesTransform( iNextCascadeIndex, finalwpos, vShadowMapTextureCoord_blend );  
         float fPercentLit_blend = 0.0f;
         vShadowMapTextureCoord_blend.z -= sbias + (fCurrentPixelDepth/20000000.0f);
         CalculatePCFPercentLit ( iNextCascadeIndex, vShadowMapTextureCoord_blend, fCurrentPixelDepth, finalwpos.xz, fPercentLit_blend );
         fShadow = lerp( fPercentLit_blend, fShadow, fBlendBetweenCascadesAmount ); 
      }
      fShadow = min ( (fShadow * ShadowStrength) , 0.85f );
    }
    float ShadowSunFactor = lerp(1.0,dot(Ln,IN.WorldNormal),SurfaceSunFactor);
    fShadow = clamp( fShadow * ShadowSunFactor * 3.5f, 0.0 , 0.85 );
    float fInvShadow = 1.0-fShadow;
    lighting.y *= fInvShadow;
	
    float4 dynamicContrib = float4(0,0,0,0);
    float4 viewspacePos = mul(IN.WPos, View);
    float conewidth = 24;
    float intensity = max(0, 1.5f - (viewspacePos.z/500.0f));
    float3 lightvector = Vn;
    float3 lightdir = float3(View._m02,View._m12,View._m22);
    float flashlight = pow(max(dot(-lightvector, lightdir),0),conewidth) * intensity * SpotFlashColor.w;   
    dynamicContrib.xyz = dynamicContrib.xyz + (diffusemap.xyz*float3(flashlight,flashlight,flashlight));
    float4 diffuseContrib = SurfColor * diffusemap * lighting.y * GlobalSurfaceIntensity;
    float fSkyFloorRatio = (1+dot(IN.WorldNormal.xyz,float3(0,1,0)))/2;
    float3 bouncelightcolor = lerp(FloorColor,SkyColor,fSkyFloorRatio) * diffusemap.xyz * 0.8;
    bouncelightcolor = bouncelightcolor + (diffusemap.xyz * 0.2);
    float4 ambContrib = float4(bouncelightcolor,1) * AmbiColor * AmbiColorOverride * 2;
    float4 result = diffuseContrib + ambContrib + dynamicContrib;
    float4 cameraPos = mul(IN.WPos, View);
    float hudfogfactor = saturate((cameraPos.z- HudFogDist.x)/(HudFogDist.y - HudFogDist.x));
    float4 hudfogresult = lerp(result,float4(HudFogColor.xyz,0),hudfogfactor*HudFogColor.w);
    hudfogresult.a = diffusemap.a * alphaoverride;    
    float alphaslice = 1.0f - min(1,max(0,IN.WPos.y - EntityEffectControl.x)/50.0f);
    hudfogresult.a = hudfogresult.a * alphaslice;
    return hudfogresult;
}

float4 DefaultPixelShaderNoShadow( vs_out IN ) : COLOR
{
    clip(IN.clip);
	float4 diff = tex2D(DiffuseSampler,IN.uv)*0.5f;
	float4 diff2 = tex2D(DiffuseSampler,float2(1-IN.uv2.x,IN.uv2.y))*0.5f;
    float4 diffusemap = diff+diff2;
    float3 Ln = normalize(IN.LightVec);
    float3 V  = (eyePos - IN.WPos);  
    float3 Vn  = normalize(V); 
    float3 Hn = normalize(Vn+Ln);
    float4 lighting = lit(dot(Ln,IN.WorldNormal),dot(Hn,IN.WorldNormal),24);
    lighting.y = lerp(0.65,lighting.y,SurfaceSunFactor);
    float4 dynamicContrib = float4(0,0,0,0);
    float4 viewspacePos = mul(IN.WPos, View);
    float conewidth = 24;
    float intensity = max(0, 1.5f - (viewspacePos.z/500.0f));
    float3 lightvector = Vn;
    float3 lightdir = float3(View._m02,View._m12,View._m22);
    float flashlight = pow(max(dot(-lightvector, lightdir),0),conewidth) * intensity * SpotFlashColor.w;   
    dynamicContrib.xyz = dynamicContrib.xyz + (diffusemap.xyz*float3(flashlight,flashlight,flashlight));
    float4 diffuseContrib = SurfColor * diffusemap * lighting.y * GlobalSurfaceIntensity;
    float fSkyFloorRatio = (1+dot(IN.WorldNormal.xyz,float3(0,1,0)))/2;
    float3 bouncelightcolor = lerp(FloorColor,SkyColor,fSkyFloorRatio) * diffusemap.xyz * 0.8;
    bouncelightcolor = bouncelightcolor + (diffusemap.xyz * 0.2);
    float4 ambContrib = float4(bouncelightcolor,1) * AmbiColor * AmbiColorOverride * 2;
    float4 result = diffuseContrib + ambContrib + dynamicContrib;
    float4 cameraPos = mul(IN.WPos, View);
    float hudfogfactor = saturate((cameraPos.z- HudFogDist.x)/(HudFogDist.y - HudFogDist.x));
    float4 hudfogresult = lerp(result,float4(HudFogColor.xyz,0),hudfogfactor*HudFogColor.w);
    hudfogresult.a = diffusemap.a * alphaoverride;    
    float alphaslice = 1.0f - min(1,max(0,IN.WPos.y - EntityEffectControl.x)/50.0f);
    hudfogresult.a = hudfogresult.a * alphaslice;
    return hudfogresult;
}

float4 blackPS(vs_out IN) : COLOR
{
   clip(IN.clip); // all shaders should receive the clip value  
   return IN.vDepth;
}

float4 mainPS_renderdepth(vs_out IN) : COLOR
{
   clip(IN.clip);
   return IN.WPos.wxyz;
}

technique Highest
{
    pass RenderDepthPixelsPass
    <
      string RenderColorTarget = "[depthtexture]";
    >
    {
        // shaders
        VertexShader = compile vs_3_0 DefaultVertexShader();
        PixelShader  = compile ps_3_0 mainPS_renderdepth();
        AlphaBlendEnable = false;
        AlphaTestEnable = false;
    }
    pass MainPass
    <
      string RenderColorTarget = "";
    >
    {
		VertexShader = compile vs_3_0 DefaultVertexShader( );
		PixelShader = compile ps_3_0 DefaultPixelShader( );
		CullMode = NONE;      
		Alphatestenable = FALSE;
		Alphablendenable = TRUE;
		ZEnable = TRUE;
		ZWriteEnable = TRUE;
    }
}

technique Medium
{
    pass RenderDepthPixelsPass
    <
      string RenderColorTarget = "[depthtexture]";
    >
    {
        // shaders
        VertexShader = compile vs_3_0 DefaultVertexShader();
        PixelShader  = compile ps_3_0 mainPS_renderdepth();
        AlphaBlendEnable = false;
        AlphaTestEnable = false;
    }
    pass MainPass
    <
      string RenderColorTarget = "";
    >
    {
		VertexShader = compile vs_3_0 DefaultVertexShader( );
		PixelShader = compile ps_3_0 DefaultPixelShaderNoShadow( );
		CullMode = NONE;      
		Alphatestenable = FALSE;
		Alphablendenable = TRUE;
		ZEnable = TRUE;
		ZWriteEnable = TRUE;
    }
}

technique Lowest
{
    pass RenderDepthPixelsPass
    <
      string RenderColorTarget = "[depthtexture]";
    >
    {
        // shaders
        VertexShader = compile vs_3_0 DefaultVertexShader();
        PixelShader  = compile ps_3_0 mainPS_renderdepth();
        AlphaBlendEnable = false;
        AlphaTestEnable = false;
    }
    pass MainPass
    <
      string RenderColorTarget = "";
    >
    {
		VertexShader = compile vs_3_0 DefaultVertexShader( );
		PixelShader = compile ps_3_0 DefaultPixelShaderNoShadow( );
		CullMode = NONE;      
		Alphatestenable = FALSE;
		Alphablendenable = TRUE;
		ZEnable = TRUE;
		ZWriteEnable = TRUE;
    }
}

technique DepthMap
{
    pass p0
    {      
        VertexShader = compile vs_3_0 DefaultVertexShader();
        PixelShader  = compile ps_3_0 blackPS();
        CullMode = CCW;
        alphafunc = greater;
        alpharef = 128;
        AlphaBlendEnable = FALSE;
        AlphaTestEnable = true;
   }
}

technique blacktextured
{
    pass P0
    {
        // shaders
        VertexShader = compile vs_3_0 DefaultVertexShader();
        PixelShader  = compile ps_3_0 blackPS();
        CullMode = ccw;
        alphafunc=greater;
        alpharef=128 ;
        AlphaBlendEnable = FALSE;
        AlphaTestEnable = true;
    }
}
