string Description = "Static Shader";
#include "settings.fx"                                                    

// PE: better poisson disk spreading.
const uint poissonDisknum = 16;
static const float2 poissonDisk[16] =
   {
      float2(0.2770745f, 0.6951455f),
      float2(0.1874257f, -0.02561589f),
      float2(-0.3381929f, 0.8713168f),
      float2(0.5867746f, 0.1087471f),
      float2(-0.3078699f, 0.188545f),
      float2(0.7993396f, 0.4595091f),
      float2(-0.09242552f, 0.5260149f),
      float2(0.3657553f, -0.5329605f),
      float2(-0.3829718f, -0.2476171f),
      float2(-0.01085108f, -0.6966301f),
      float2(0.8404155f, -0.3543923f),
      float2(-0.5186161f, -0.7624033f),
      float2(-0.8135794f, 0.2328489f),
      float2(-0.784665f, -0.2434929f),
      float2(0.9920505f, 0.0855163f),
      float2(-0.687256f, 0.6711345f)
   };

// shadow mapping
matrix          m_mShadow;
float4          m_vCascadeOffset[8];
float4          m_vCascadeScale[8];
int             m_nCascadeLevels;
float           m_fMinBorderPadding;     
float           m_fMaxBorderPadding;
float           m_fShadowBiasFromGUI;  // A shadow map offset to deal with self shadow artifacts.  
float           m_fCascadeBlendArea; // Amount to overlap when blending between cascades.
float           m_fTexelSize; 
float           m_fCascadeFrustumsEyeSpaceDepths[8];
float3          m_vLightDir;

float ShadowStrength
<    string UIName =  "ShadowStrength";    
> = {1.0f};

float SurfaceSunFactor
<    string UIName =  "SurfaceSunFactor";    
> = {1.0f};

float GlobalSpecular
<    string UIName =  "GlobalSpecular";    
> = {0.5f};

float GlobalSurfaceIntensity
<    string UIName =  "GlobalSurfaceIntensity";    
> = {1.0f};

// standard constants
float4x4 World : World;
float4x4 WorldInverse : WorldInverse;
float4x4 WorldIT : WorldInverseTranspose;
float4x4 WorldView : WorldView;
float4x4 WorldViewProjection : WorldViewProjection;
float4x4 View : View;
float4x4 ViewInverse : ViewInverse;
float4x4 ViewIT : ViewInverseTranspose;
float4x4 ViewProjection : ViewProjection;
float4x4 Projection : Projection;
float4 eyePos : CameraPosition;
float m_fClippingOnState = 1;

/**************VALUES PROVIDED FROM FPSC - NON TWEAKABLE**************************************/

float4 clipPlane : ClipPlane;  //cliplane for water plane

float4 LightSource
<   string UIType = "Fixed Light Direction";
> = {-1.0f, -1.0f, -1.0f, 1.0f};

//SpotFlash Values from FPSC (SpotFlashPos.w is carrying the spotflash fadeout value + SpotFlashColor.w carries FlashLightStrength)
float4 SpotFlashPos;
float4 SpotFlashColor;

float4 FogColor : Diffuse
<   string UIName =  "Fog Color";    
> = {0.0f, 0.0f, 0.0f, 0.0000001f};

float4 HudFogColor : Diffuse
<   string UIName =  "Hud Fog Color";    
> = {0.0f, 0.0f, 0.0f, 0.0000001f};

float4 HudFogDist : Diffuse
<   string UIName =  "Hud Fog Dist";    
> = {1.0f, 0.0f, 0.0f, 0.0000001f};

float4 AmbiColorOverride
<    string UIName =  "AmbiColorOverride";    
> = {1.0f, 1.0f, 1.0f, 1.0f};

float4 AmbiColor : Ambient
<    string UIName =  "AmbiColor";    
> = {0.1f, 0.1f, 0.1f, 1.0f};

float4 SurfColor : Diffuse
<    string UIName =  "SurfColor";    
> = {1.0f, 1.0f, 1.0f, 1.0f};

float SpecularOverride = 1.0f;

float4 SkyColor : Diffuse
<    string UIName =  "SkyColor";    
> = {1.0, 1.0, 1.0, 1.0f};

float4 FloorColor : Diffuse
<    string UIName =  "FloorColor";    
> = {1.0, 1.0, 1.00, 1.0f};

float alphaoverride  : alphaoverride;

float4 EntityEffectControl
<    string UIName =  "EntityEffectControl";    
> = {0.0f, 0.0f, 0.0f, 0.0f};

//Shader Variables pulled from FPI scripting 
float4 ShaderVariables : ShaderVariables
<    string UIName =  "Shader Variables";    
> = {1.0f, 1.0f, 1.0f, 1.0f};

//Supports dynamic lights (using CalcLighting function)
float4 g_lights_data;
float4 g_lights_pos0;
float4 g_lights_pos1;
float4 g_lights_pos2;
float4 g_lights_atten0;
float4 g_lights_atten1;
float4 g_lights_atten2;
float4 g_lights_diffuse0;
float4 g_lights_diffuse1;
float4 g_lights_diffuse2;

// RENDERCOLORTARGET simply indicates that we are using an RT (depth texture at bottom)
texture DiffuseMap : RENDERCOLORTARGET
<
    string Name = "D.tga";
    string type = "2D";
>;

texture OcclusionMap : DiffuseMap
<
    string Name = "O.tga";
    string type = "2D";
>;

texture NormalMap : DiffuseMap
<
    string Name = "N.tga";
    string type = "2D";
>;

texture SpecularMap : DiffuseMap
<
    string Name = "S.tga";
    string type = "2D";
>;
texture VegShadowTex : DiffuseMap
<
    string Name = "D.tga";
    string type = "2D";
>;
texture DynTerShaMap : DiffuseMap
<
    string Name = "D.tga";
    string type = "2D";
>;
texture IlluminationMap : DiffuseMap
<
    string Name = "I.tga";
    string type = "2D";
>;

texture DepthMapTX1 : DiffuseMap
<
    string Name = "DEPTH1.tga";
    string type = "2D";
>;
texture DepthMapTX2 : DiffuseMap
<
    string Name = "DEPTH1.tga";
    string type = "2D";
>;
texture DepthMapTX3 : DiffuseMap
<
    string Name = "DEPTH1.tga";
    string type = "2D";
>;
texture DepthMapTX4 : DiffuseMap
<
    string Name = "D.tga";
    string type = "2D";
>;

//Diffuse Texture _D
sampler2D DiffuseSampler = sampler_state
{
    Texture   = <DiffuseMap>;
    MipFilter = LINEAR;
    MinFilter = ANISOTROPIC;
    MagFilter = LINEAR;
};

//Lightmap in static entities, occlusion map in dynamic entities _O
sampler2D OccSampler = sampler_state
{
    Texture   = <OcclusionMap>;
    MipFilter = LINEAR;
    MinFilter = ANISOTROPIC;
    MagFilter = LINEAR;
};

//Effect Texture _N 
sampler2D NormalSampler = sampler_state
{
    Texture   = <NormalMap>;
    MipFilter = LINEAR;
    MinFilter = ANISOTROPIC;
    MagFilter = LINEAR;
};

//Effect Texture _S 
sampler2D SpecSampler = sampler_state
{
    Texture   = <SpecularMap>;
    MipFilter = LINEAR;
    MinFilter = ANISOTROPIC;
    MagFilter = LINEAR;
};

sampler VegShadowSamp = sampler_state
{
   Texture = <VegShadowTex>;
   MinFilter = Linear; 
   MagFilter = Linear; 
   MipFilter = Linear;
   AddressU = clamp; 
   AddressV = clamp;
};

sampler2D DynTerShaSampler = sampler_state
{
    Texture   = <DynTerShaMap>;
    MipFilter = LINEAR;
    MinFilter = ANISOTROPIC;
    MagFilter = LINEAR;
    AddressU = Clamp;
    AddressV = Clamp;
};

sampler2D IllumSampler = sampler_state
{
    Texture   = <IlluminationMap>;
    MipFilter = LINEAR;
    MinFilter = ANISOTROPIC;
    MagFilter = LINEAR;
};

sampler2D DepthMap1 = sampler_state
{
   Texture = <DepthMapTX1>;   
    MinFilter = LINEAR;
    MagFilter = LINEAR;
    AddressU = Clamp;
    AddressV = Clamp;
};
sampler2D DepthMap2 = sampler_state
{
   Texture = <DepthMapTX2>;   
    MinFilter = LINEAR;
    MagFilter = LINEAR;
    AddressU = Clamp;
    AddressV = Clamp;
};
sampler2D DepthMap3 = sampler_state
{
   Texture = <DepthMapTX3>;   
    MinFilter = LINEAR;
    MagFilter = LINEAR;
    AddressU = Clamp;
    AddressV = Clamp;
};
sampler2D DepthMap4 = sampler_state
{
   Texture = <DepthMapTX4>;   
    MinFilter = LINEAR;
    MagFilter = LINEAR;
    AddressU = Wrap;
    AddressV = Wrap;
    //AddressU = Border;
    //AddressV = Border;
   //BorderColor = 0xFFFFFFFF;
};

struct appdata {
    float4 Position   : POSITION;
    float2 UV      : TEXCOORD0;
    float2 UV2      : TEXCOORD1;
    float4 Normal   : NORMAL;
    float4 Tangent   : TANGENT0;
    float4 Binormal   : BINORMAL0;
};

/*data passed to pixel shader*/
struct vertexOutput
{
    float4 Position     : POSITION;
    float2 TexCoord     : TEXCOORD0;
    float2 TexCoord2    : TEXCOORD1;
    float3 LightVec       : TEXCOORD2;
    float3 WorldNormal   : TEXCOORD3;
    float3 WorldTangent   : TEXCOORD4;
    float3 WorldBinorm   : TEXCOORD5;
    float4 WPos         : TEXCOORD7;
    float  clip         : TEXCOORD8;
    float2 vDepth       : TEXCOORD9;
};

struct vertexOutput_low
{
    float4 Position     : POSITION;
    float2 TexCoord     : TEXCOORD0;
    float2 TexCoord2    : TEXCOORD1;
    float3 LightVec       : TEXCOORD2;
    float3 WorldNormal   : TEXCOORD3;
    float4 WPos         : TEXCOORD4;
   float2 vegshadowuv  : TEXCOORD5;
    float  clip         : TEXCOORD6;
    float  vDepth       : TEXCOORD7;
};

/****** helper functions for shadow mapping*****/

void ComputeCoordinatesTransform( in int iCascadeIndex,
                                  in float4 InterpolatedPosition,
                                  in out float4 vShadowTexCoord ) 
{
   float4 vLightDot = mul ( InterpolatedPosition, m_mShadow );
    vLightDot *= m_vCascadeScale[iCascadeIndex];
    vLightDot += m_vCascadeOffset[iCascadeIndex];
   vShadowTexCoord.xyz = vLightDot.xyz;
} 


       
float texture2DCompare(sampler2D depths, float2 uv, float compare){
    float depth = tex2D(depths, uv).r;
    return step(compare, depth);
}
float PCF5x5(sampler2D depths, float2 size, float2 uv, float compare){
    float result = 0.0;
    for(int x=-2; x<=2; x++){
        for(int y=-2; y<=2; y++){
            float2 off = float2(x,y)/size;
            result += texture2DCompare(depths, uv+off, compare);
        }
    }
    return result/25.0;
}
float PCF5x3(sampler2D depths, float2 size, float2 uv, float compare){
    float result = 0.0;
    for(int x=-1; x<=1; x++){
        for(int y=-2; y<=2; y++){
            float2 off = float2(x,y)/size;
            result += texture2DCompare(depths, uv+off, compare);
        }
    }
    return result/15.0;
}
float PCF4x4(sampler2D depths, float2 size, float2 uv, float compare){
    float result = 0.0;
    float2 texsize = 1.0/size;
    
    for(float x=-1.5; x<=1.5; x+= 1.0){
        for(float y=-1.5; y<=1.5; y+=1.0){
            float2 off = float2(x,y) * texsize;
            result += texture2DCompare(depths, uv+off, compare);
        }
    }
    return result/16.0;
}
float PCF4x4rand(sampler2D depths, float2 size, float2 uv, float compare, float2 mwpos, float pradius){
    float result = 0.0;
    float2 texsize = 1.0/size;
    int runs = 0;
    for(float x=-1.5; x<=1.5; x+= 1.0){
        for(float y=-1.5; y<=1.5; y+=1.0){
            //float2 o = 1.0-( fmod(floor(mwpos*(y*x)), 100.0) * 0.01 );
            //int pidx = clamp( floor(fmod(floor(mwpos*(y+x)), poissonDisknum)) , 0 , poissonDisknum );
            //mwpos += float2(x+y,y+x);
            //float2 o = poissonDisk[ clamp(runs,0,poissonDisknum) ];
            //float2 off = (o+float2(x,y)) * texsize;
            float2 off = ( poissonDisk[ clamp(runs,0,poissonDisknum) ] * pradius ) * texsize;

            result += texture2DCompare(depths, uv+off, compare);
            runs++;
        }
    }
    result += texture2DCompare(depths, uv, compare);
    result += texture2DCompare(depths, uv+( float2(0.55,0.55) * texsize ), compare);

    return result/18.0;
}
float PCF3x3rand(sampler2D depths, float2 size, float2 uv, float compare, float2 mwpos, float pradius){
    float result = 0.0;
    float2 texsize = 1.0/size;
    int runs = 0;
    for(float x=-1.0; x<=1.0; x+= 1.0){
        for(float y=-1.0; y<=1.0; y+=1.0){
            //float2 o = 1.0-( fmod(floor(mwpos*(y*x)), 100.0) * 0.01 );
            //int pidx = clamp( floor(fmod(floor(mwpos*(y+x)), poissonDisknum)) , 0 , poissonDisknum );
            //mwpos += float2(x+y,y+x);
            //float2 o = poissonDisk[ clamp(runs,0,poissonDisknum) ];
            //float2 off = (o+float2(x,y)) * texsize;
            float2 off = ( poissonDisk[ clamp(runs,0,poissonDisknum) ] * pradius ) * texsize;

            result += texture2DCompare(depths, uv+off, compare);
            runs++;
        }
    }
//    result += texture2DCompare(depths, uv, compare);
//    result += texture2DCompare(depths, uv+( float2(0.55,0.55) * texsize ), compare);

    return result/9.0;
}
float PCF3x3(sampler2D depths, float2 size, float2 uv, float compare){
    float result = 0.0;
    for(int x=-1; x<=1; x++){
        for(int y=-1; y<=1; y++){
            float2 off = float2(x,y)/size;
            result += texture2DCompare(depths, uv+off, compare);
        }
    }
    return result/9.0;
}

void CalculatePCFPercentLit ( in int iCurrentCascadeIndex,
                       in float4 vShadowTexCoord, 
                       in float fCurrentDepth,
                       in float2 mwpos,                       
                              out float fPercentLit ) 
{
   fPercentLit = 0.0f;
   float2 fTS = float2(1.0f/SHADOWTEXELX,1.0f/SHADOWTEXELY);
   float2 tsize = float2( SHADOWTEXELX ,SHADOWTEXELY );

   if ( iCurrentCascadeIndex==0 )
   {
#ifdef DEBUGSHADOW
        fPercentLit += vShadowTexCoord.z > tex2D(DepthMap1,float2(vShadowTexCoord.x,vShadowTexCoord.y) ).x ? 1.0f : 0.0f;
#else    
#ifdef BETTERSHADOWS
        float2 o = ( fmod(floor(mwpos), 10.0) * 0.1 )*fTS;
        fPercentLit = 1.0-PCF4x4rand(DepthMap1, tsize, vShadowTexCoord.xy, vShadowTexCoord.z-0.00003f,mwpos , 2.35 );
        //fPercentLit = 1.0-PCF4x4(DepthMap1, tsize, vShadowTexCoord.xy, vShadowTexCoord.z-0.00003f );
        //fPercentLit = vShadowTexCoord.z > tex2D(DepthMap1,float2(vShadowTexCoord.x,vShadowTexCoord.y) ).x ? 1.0f : 0.0f;
#else
        // test dither with random offsets.
        float2 o = fmod(floor(mwpos), 1.50) * 0.25;
        fPercentLit += vShadowTexCoord.z > tex2D(DepthMap1,float2(vShadowTexCoord.x,vShadowTexCoord.y)+(  (float2(-0.75, 0.75) + o) *fTS)  ).x ? 1.0f : 0.0f;
        fPercentLit += vShadowTexCoord.z > tex2D(DepthMap1,float2(vShadowTexCoord.x,vShadowTexCoord.y)+(  (float2( 0.25, 0.75) + o) *fTS)  ).x ? 1.0f : 0.0f;
        fPercentLit += vShadowTexCoord.z > tex2D(DepthMap1,float2(vShadowTexCoord.x,vShadowTexCoord.y)+(  (float2(-0.75,-0.25) + o) *fTS)  ).x ? 1.0f : 0.0f;
        fPercentLit += vShadowTexCoord.z > tex2D(DepthMap1,float2(vShadowTexCoord.x,vShadowTexCoord.y)+(  (float2( 0.25,-0.25) + o) *fTS)  ).x ? 1.0f : 0.0f;
        fPercentLit += vShadowTexCoord.z > tex2D(DepthMap1,float2(vShadowTexCoord.x,vShadowTexCoord.y)+(  (float2( 0.75,-0.75) + o) *fTS)  ).x ? 1.0f : 0.0f;
        fPercentLit *= 0.20;
#endif
#endif
   }
   else
   {
      if ( iCurrentCascadeIndex==1 )
      {
         // test dither with random offsets.
         // cords on different shadow maps dont match 100% so adjust ( -0.00003f )
#ifdef DEBUGSHADOW
        fPercentLit += vShadowTexCoord.z > tex2D(DepthMap2,float2(vShadowTexCoord.x,vShadowTexCoord.y) ).x ? 1.0f : 0.0f;
#else    
#ifdef BETTERSHADOWS
           float2 o = (fmod(floor(mwpos), 1.50) * 0.25)*fTS;
           fPercentLit = 1.0-PCF4x4rand(DepthMap2, float2( SHADOWTEXELX * 0.60 ,SHADOWTEXELY * 3.0  )  , vShadowTexCoord.xy , vShadowTexCoord.z-0.00004f,mwpos , 1.65 );
           //fPercentLit = vShadowTexCoord.z-0.00003f > tex2D(DepthMap2,float2(vShadowTexCoord.x,vShadowTexCoord.y) ).x ? 1.0f : 0.0f;
           fPercentLit *= 0.90f;
#else
         float2 o = fmod(floor(mwpos), 1.50) * 0.25;
         fPercentLit += vShadowTexCoord.z-0.00003f > tex2D(DepthMap2,float2(vShadowTexCoord.x,vShadowTexCoord.y)+( (float2( 0.25, 0.75) + o) *fTS)).x ? 1.0f : 0.0f;
         fPercentLit += vShadowTexCoord.z-0.00003f > tex2D(DepthMap2,float2(vShadowTexCoord.x,vShadowTexCoord.y)+( (float2(-0.75,-0.25) + o) *fTS)).x ? 1.0f : 0.0f;
         fPercentLit *= 0.5;
#endif
#endif
      }
      else
      {
         if ( iCurrentCascadeIndex==2 )
         {
#ifdef DEBUGSHADOW
        fPercentLit += vShadowTexCoord.z > tex2D(DepthMap3,float2(vShadowTexCoord.x,vShadowTexCoord.y) ).x ? 1.0f : 0.0f;
#else    
#ifdef BETTERSHADOWS
            //float2 o = fmod(floor(mwpos), 1.50) * 0.25;
            //fPercentLit = 1.0-PCF3x3rand(DepthMap3, float2( SHADOWTEXELX * 1.2 ,SHADOWTEXELY * 3.5  )  , float2(vShadowTexCoord.x,vShadowTexCoord.y) , vShadowTexCoord.z,mwpos , 1.15 );
            //fPercentLit *= 0.85;

            fPercentLit += vShadowTexCoord.z-0.000044f > tex2D(DepthMap3,float2(vShadowTexCoord.x,vShadowTexCoord.y+0.000010f  )).x ? 1.0f : 0.0f;
            fPercentLit += vShadowTexCoord.z-0.000044f > tex2D(DepthMap3,float2(vShadowTexCoord.x,vShadowTexCoord.y+0.000010f  )+( (float2( 0.55, 0.0) ) *fTS)).x ? 1.0f : 0.0f;
            ////fPercentLit += vShadowTexCoord.z-0.000009f > tex2D(DepthMap3,float2(vShadowTexCoord.x,vShadowTexCoord.y+0.00020f)+( (float2(-0.45, 0.0) ) *fTS)).x ? 1.0f : 0.0f;
            fPercentLit *= 0.30f;
#else
            fPercentLit += vShadowTexCoord.z-0.00009f > tex2D(DepthMap3,float2(vShadowTexCoord.x,vShadowTexCoord.y)).x ? 1.0f : 0.0f;
            fPercentLit *= 0.5f;
#endif
#endif
         }
         else
         {
            if ( iCurrentCascadeIndex==3 && vShadowTexCoord.z<1.0 )
            {
#ifdef DEBUGSHADOW
        fPercentLit += vShadowTexCoord.z > tex2D(DepthMap4,float2(vShadowTexCoord.x,vShadowTexCoord.y) ).x ? 0.5f : 0.0f;
#else    
#ifdef BETTERSHADOWS
//               fPercentLit += vShadowTexCoord.z-0.000012f > tex2D(DepthMap4,float2(vShadowTexCoord.x,vShadowTexCoord.y+0.00020f  )).x ? 0.5f : 0.0f;
               fPercentLit += vShadowTexCoord.z-0.00016f > tex2D(DepthMap4,float2(vShadowTexCoord.x,vShadowTexCoord.y+0.00020f )).x ? 0.40f : 0.0f;
#else
               // give artifacts in distance , dont look like the others , so set to 0 for now.
               //fPercentLit += vShadowTexCoord.z-0.00012f > tex2D(DepthMap4,float2(vShadowTexCoord.x,vShadowTexCoord.y)).x ? 0.10f : 0.0f;
               //fPercentLit *= 0.25f;
               //fPercentLit = 0.0;
#endif
#endif
            }
         }
      }
   }                                                                             
   float fFadeOutLastCascade = saturate(1.0-((fCurrentDepth- (m_fCascadeFrustumsEyeSpaceDepths[2]+2800) )/ 600 ));
   fPercentLit *= fFadeOutLastCascade;

}


void CalculateBlendAmountForInterval ( in int iCurrentCascadeIndex, 
                                       in out float fPixelDepth, 
                                       in out float fCurrentPixelsBlendBandLocation,
                                       out float fBlendBetweenCascadesAmount ) 
{
   // Calculate amount to blend between two cascades and the band where blending will occure.
    // We need to calculate the band of the current shadow map where it will fade into the next cascade.
    // We can then early out of the expensive PCF for loop. 
    float fBlendInterval = m_fCascadeFrustumsEyeSpaceDepths[ iCurrentCascadeIndex ];
   
    int fBlendIntervalbelowIndex = min(0, iCurrentCascadeIndex-1);
   if ( fBlendIntervalbelowIndex>1 )
   {
      fPixelDepth -= m_fCascadeFrustumsEyeSpaceDepths[ fBlendIntervalbelowIndex ];
      fBlendInterval -= m_fCascadeFrustumsEyeSpaceDepths[ fBlendIntervalbelowIndex ];
    }
   
    // The current pixel's blend band location will be used to determine when we need to blend and by how much.
    fCurrentPixelsBlendBandLocation = 1.0f - (fPixelDepth / fBlendInterval);
   
    // The fBlendBetweenCascadesAmount is our location in the blend band.
    fBlendBetweenCascadesAmount = fCurrentPixelsBlendBandLocation / m_fCascadeBlendArea;
}

/*******Main Vertex Shader***************************/

vertexOutput mainVS_highest(appdata IN)   
{
   vertexOutput OUT;
   
   float4 worldSpacePos = mul(IN.Position, World);
   OUT.WPos =   worldSpacePos; 
      
   OUT.WorldNormal = normalize(mul(IN.Normal, WorldIT).xyz);
//   OUT.WorldTangent = normalize(mul(IN.Tangent, WorldIT).xyz);
   // PE: something is wrong with tangent. lee: try to display it directly in fragment,
   // on some angles it gives different result on a flat surface ?
   //OUT.WorldTangent = normalize(mul(IN.Tangent, WorldIT).xyz);
   float3 c1 = cross(OUT.WorldNormal, float3(0.0, 0.0, 1.0)); 
   float3 c2 = cross(OUT.WorldNormal, float3(0.0, 1.0, 0.0)); 
   if (length(c1) > length(c2)) {
     OUT.WorldTangent = c1;   
   } else {
     OUT.WorldTangent = c2;   
   }
   OUT.WorldTangent = normalize(OUT.WorldTangent); 

   OUT.WorldBinorm = normalize(mul(IN.Binormal, WorldIT).xyz);
      
   OUT.LightVec = normalize(LightSource);
   OUT.Position = mul(IN.Position, WorldViewProjection);
   OUT.TexCoord = IN.UV; 
   OUT.TexCoord2 = IN.UV2; 
                
   // all shaders should send the clip value to the pixel shader (for refr/refl)                                                                     
   OUT.clip = dot(worldSpacePos, clipPlane);                                                                      
     
   // SHADOW MAPPING - world position and projected depth (for cascade distance calc)
   OUT.vDepth.x = mul( IN.Position, WorldViewProjection ).z; 
   // defeat projective aliasing by detecting when parallel to light direction
   OUT.vDepth.y = max(0,abs(dot(OUT.LightVec, OUT.WorldNormal))-0.25f)*1.333f;
   
    return OUT;
}

vertexOutput_low mainVS_lowest(appdata IN)   
{
   vertexOutput_low OUT;
   float4 worldSpacePos = mul(IN.Position, World);
   OUT.WPos =   worldSpacePos; 
   OUT.WorldNormal = normalize(mul(IN.Normal, WorldIT).xyz);
   OUT.LightVec = normalize(LightSource);
   OUT.Position = mul(IN.Position, WorldViewProjection);
   OUT.TexCoord  = IN.UV; 
   OUT.TexCoord2 = IN.UV2; 
   OUT.vegshadowuv = float2(worldSpacePos.x/51200.0f,worldSpacePos.z/51200.0f);
   OUT.clip = dot(worldSpacePos, clipPlane);                                                                      
   OUT.vDepth = mul( IN.Position, WorldViewProjection ).z; 
    return OUT;
}

float4 CalcSpotFlash( float3 worldNormal, float3 worldPos )
{
    float4 output = (float4)0.0;
    float3 toLight = (SpotFlashPos.xyz - worldPos.xyz);
    float3 lightDir = normalize( toLight );
    float lightDist = length( toLight );
    
    float MinFalloff = 100;  //falloff start distance
    float LinearFalloff = 1;
    float ExpFalloff = .005;  // 1/200
    float fSpotFlashPosW = clamp(0,1,SpotFlashPos.w);
    
    //classic attenuation - but never actually reaches zero
    float fAtten = 1.0/(MinFalloff + (LinearFalloff*lightDist)+(ExpFalloff*lightDist*lightDist));
    output += (float4(SpotFlashColor.xyz,1)) *fAtten * (fSpotFlashPosW) * max(0,dot(worldNormal,lightDir));
        
    return output;
}

float4 CalcLighting(float3 Nb, float3 worldPos, float3 Vn, float4 diffusemap, float4 specmap)
{           
    float4 output = (float4)0.0;
#ifdef VERSION114
   if ( g_lights_data.x == 0 ) return output;
#endif    
    // light 0
    float3 toLight = g_lights_pos0.xyz - worldPos;
    float lightDist = length( toLight );

    float fAtten;
    float3 lightDir;
    float3 halfvec;
    float4 lit0;
//    if( lightDist < g_lights_pos0.w*4.0 ) {  
       float4 local_lights_atten0 = float4(1.0, 1.0/g_lights_pos0.w, 1.0/(g_lights_pos0.w*g_lights_pos0.w), 0.0);
       //float fAtten = 1.0/dot( g_lights_atten0, float4(1,lightDist,lightDist*lightDist,0) ); // NVIDIA 361 bug
       fAtten = 1.0/dot( local_lights_atten0, float4(1,lightDist,lightDist*lightDist,0) );
       lightDir = normalize( toLight );
       halfvec = normalize(Vn + lightDir);
       lit0 = lit(dot(lightDir,Nb),dot(halfvec,Nb),24); 
       output+= (lit0.y *g_lights_diffuse0 * fAtten * diffusemap) + (lit0.z * g_lights_diffuse0 * fAtten *specmap );   
//    }   
    // light 1
    toLight = g_lights_pos1.xyz - worldPos;
    lightDist = length( toLight );
//    if( lightDist < g_lights_pos1.w*4.0 ) {  
       float4 local_lights_atten1 = float4(1.0, 1.0/g_lights_pos1.w, 1.0/(g_lights_pos1.w*g_lights_pos1.w), 0.0);
       //fAtten = 1.0/dot( g_lights_atten1, float4(1,lightDist,lightDist*lightDist,0) ); // NVIDIA 361 bug
       fAtten = 1.0/dot( local_lights_atten1, float4(1,lightDist,lightDist*lightDist,0) );
       lightDir = normalize( toLight );
       halfvec = normalize(Vn + lightDir);
       lit0 = lit(dot(lightDir,Nb),dot(halfvec,Nb),24); 
       output+= (lit0.y *g_lights_diffuse1 * fAtten * diffusemap) + (lit0.z * g_lights_diffuse1 * fAtten *specmap );   
//    }   
    // light 2
    toLight = g_lights_pos2.xyz - worldPos;
    lightDist = length( toLight );
//    if( lightDist < g_lights_pos2.w*4.0 ) {  
       float4 local_lights_atten2 = float4(1.0, 1.0/g_lights_pos2.w, 1.0/(g_lights_pos2.w*g_lights_pos2.w), 0.0);
       //fAtten = 1.0/dot( g_lights_atten2, float4(1,lightDist,lightDist*lightDist,0) ); // NVIDIA 361 bug
       fAtten = 1.0/dot( local_lights_atten2, float4(1,lightDist,lightDist*lightDist,0) );
       lightDir = normalize( toLight );
       halfvec = normalize(Vn + lightDir);
       lit0 = lit(dot(lightDir,Nb),dot(halfvec,Nb),24); 
       output+= (lit0.y *g_lights_diffuse2 * fAtten * diffusemap) + (lit0.z * g_lights_diffuse2 * fAtten *specmap );   
//    }   
    return output;
}

float GetShadowBias ( int iCascadeIndex, float fDirectSunBias, float fPixelDepth )
{
   float sbias = ADJUSTSHADOW;
   if ( iCascadeIndex == 0 ) sbias = sbias * (20.0f/50.0f); //(GlobalSpecular*2.0f); 
   if ( iCascadeIndex == 1 ) sbias = sbias * (2.0f/50.0f);
   if ( iCascadeIndex == 2 ) sbias = sbias * (20.0f/50.0f);
   if ( iCascadeIndex == 3 ) sbias = sbias * (70.0f/50.0f);
   sbias = sbias * pow(fDirectSunBias,2);
   sbias = sbias + (fPixelDepth/20000000.0f);
   return sbias;
}

float4 mainPS_highest(vertexOutput IN) : COLOR
{
   // clip
    clip(IN.clip);
   
   // source textures
    float4 diffusemap = tex2D(DiffuseSampler,IN.TexCoord.xy);
    float3 normalmap = tex2D(NormalSampler,IN.TexCoord.xy) * 2 - 1;
    float4 specmap = tex2D(SpecSampler,IN.TexCoord.xy);
    float4 illummap = tex2D(IllumSampler,IN.TexCoord.xy);
   
   // work out normal from normmap/tangent/binormal
#ifdef DISABLELIGHTMAPNORMALS
    float3 Ln = (float3(0,1,0)); // 011215 - baked lights are everywhere! (IN.LightVec);
#else
    float3 Ln = normalize(IN.LightVec);
#endif
    float3 Nn = (IN.WorldNormal);
    float3 Tn = (IN.WorldTangent);
    float3 Bn = (IN.WorldBinorm);
    float3 Nb = (normalmap.z * Nn) + ((normalmap.x * Tn + normalmap.y * Bn));
    Nb = normalize(Nb);
   
   // lighting
    float3 V  = (eyePos - IN.WPos);  
    float3 Vn  = normalize(V); 
    float3 Hn = normalize(Vn+Ln);
    float4 lighting = lit(dot(Ln,Nb),dot(Hn,Nb),24);
   lighting.y = lerp(0.65,lighting.y,SurfaceSunFactor);

   // choose between surface lighting and lightmap based on shadow strength
   float4 rawlightmap = tex2D(OccSampler,IN.TexCoord2.xy);
           
   // remove lightmapper blur artifacts.
   rawlightmap = clamp(rawlightmap,0.265,1.0);

   // produced final light-color
   float4 fLightOfShadow = max(min(float4((rawlightmap.xyz-0.25f)*2,1),1),0); // 0.25=BLACK 0.75=WHITE -> 0-1 range
   //fLightOfShadow = clamp(fLightOfShadow,0.2,1.0);
   // work out any light difference from bump map
   float4 flatlighting = lit(dot(Ln,IN.WorldNormal),dot(Hn,IN.WorldNormal),24);
   flatlighting.y = lerp(0.65,flatlighting.y,SurfaceSunFactor);
   fLightOfShadow = fLightOfShadow + (lighting.y-flatlighting.y);
   
   // shadow mapping code
   float fShadow = 0.0f;
   int iCurrentCascadeIndex=3;
   float4 vShadowMapTextureCoord = 0.0f;
   if ( ShadowStrength > 0.0f )
   {
      // reset shadow vars
      float fPercentLit = 0.0f;

      // the interval based selection technique compares the pixel's depth against the frustum's cascade divisions.
      float fCurrentPixelDepth;
      fCurrentPixelDepth = IN.vDepth.x;
      if ( fCurrentPixelDepth < m_fCascadeFrustumsEyeSpaceDepths[2] ) iCurrentCascadeIndex = 2;
      if ( fCurrentPixelDepth < m_fCascadeFrustumsEyeSpaceDepths[1] ) iCurrentCascadeIndex = 1;
      if ( fCurrentPixelDepth < m_fCascadeFrustumsEyeSpaceDepths[0] ) iCurrentCascadeIndex = 0;

      // repeat text coord calculations for the next cascade - the next cascade index is used for blurring between maps.
      int iNextCascadeIndex = 1;
      iNextCascadeIndex = min ( 4 - 1, iCurrentCascadeIndex + 1 ); 
      float fBlendBetweenCascadesAmount = 1.0f;
      float fCurrentPixelsBlendBandLocation = 1.0f;
      CalculateBlendAmountForInterval ( iCurrentCascadeIndex, fCurrentPixelDepth, 
         fCurrentPixelsBlendBandLocation, fBlendBetweenCascadesAmount );

      // calculate shadowmaptexture view space
      float4 finalwpos = IN.WPos;

      // work out texture coordinate into specified shadow map
      ComputeCoordinatesTransform( iCurrentCascadeIndex, finalwpos, vShadowMapTextureCoord );    

//#ifndef DEBUGSHADOW
      // offset shadow pixel depth textcord based on light direction. 
      float sbias = lerp(ADJUSTSHADOW,0.000115f,clamp( dot(IN.WorldNormal,Ln) ,0.0,1.0));
      vShadowMapTextureCoord.z -= sbias + (fCurrentPixelDepth/20000000.0f);
      //float fDirectSunBias = clamp ( dot(IN.WorldNormal,Ln), 0.0, 1.0 );
      //float sbias = GetShadowBias(iCurrentCascadeIndex,fDirectSunBias,fCurrentPixelDepth);
      //vShadowMapTextureCoord.z -= sbias;
//#endif

      // work out how much shadow
      CalculatePCFPercentLit ( iCurrentCascadeIndex, vShadowMapTextureCoord, fCurrentPixelDepth, finalwpos.xz, fShadow );
#ifdef DEBUGSHADOW
      fCurrentPixelsBlendBandLocation = 999999.0;
#endif
                         
      if( fCurrentPixelsBlendBandLocation < m_fCascadeBlendArea ) 
      {  
         // the current pixel is within the blend band.
         // Repeat text coord calculations for the next cascade. 
         // The next cascade index is used for blurring between maps.
         float4 vShadowMapTextureCoord_blend = 0.0f;
         ComputeCoordinatesTransform( iNextCascadeIndex, finalwpos, vShadowMapTextureCoord_blend );  
      
         // the current pixel is within the blend band.
         float fPercentLit_blend = 0.0f;
         vShadowMapTextureCoord_blend.z -= sbias + (fCurrentPixelDepth/20000000.0f);

//       if ( iNextCascadeIndex < 4 )
//       {
//          ComputeCoordinatesTransform( iNextCascadeIndex, finalwpos, vShadowMapTextureCoord_blend );  
////#ifndef DEBUGSHADOW
//           sbias = GetShadowBias(iNextCascadeIndex,fDirectSunBias,fCurrentPixelDepth);
//           vShadowMapTextureCoord_blend.z -= sbias;
//           //vShadowMapTextureCoord_blend.z -= sbias + (fCurrentPixelDepth/20000000.0f);
////#endif           
          CalculatePCFPercentLit ( iNextCascadeIndex, vShadowMapTextureCoord_blend, fCurrentPixelDepth, finalwpos.xz, fPercentLit_blend );
 //      }
                           
         // Blend the two calculated shadows by the blend amount.
         fShadow = lerp( fPercentLit_blend, fShadow, fBlendBetweenCascadesAmount ); 
      }
      
      // finally modulate shadow with strength (IN.vDepth.y ensures no projective aliasing!)
      // 291115 - projectile aliasing solved - can just cap shadow max to match terrain shadow technique
      //fShadow = min ( fShadow * ShadowStrength, 0.7f );
      //PE: above dont work on dynamic shadows on lightmapped objects. ?
      fShadow *= ShadowStrength;
   }  
       
   // inverse to modulate surface lighting (and only shadow surfaces facing sunlight)
   float ShadowSunFactor = lerp(1.0,dot(Ln,IN.WorldNormal),SurfaceSunFactor);
   fShadow = fShadow * ShadowSunFactor * 3.5f; // PE: Let it look like baked shadows.
   fLightOfShadow = min(fLightOfShadow,clamp(1-fShadow,0.0,0.85));
   
    // dynamic lighting
    float4 spotflashlighting = CalcSpotFlash (Nb,IN.WPos.xyz);   
    float4 dynamicContrib = CalcLighting (Nb,IN.WPos.xyz,Vn,diffusemap,specmap) + spotflashlighting; 
   
   // 310315 - make dynamic shadow more pronounced by having it block dynamic lights
   // PE: Dynamic is already selected in fLightOfShadow.
   // PE: This way we select baked or dynamic shadow but dont mix them.
   //dynamicContrib = dynamicContrib * (1-fShadow);
  
   // flash light system (flash light control carried in SpotFlashColor.w )
   float conewidth = 24;
    float4 viewspacePos = mul(IN.WPos, View);
    float intensity = max(0, 1.5f - (viewspacePos.z/500.0f));
   float3 lightvector = Vn;
    float3 lightdir = float3(View._m02,View._m12,View._m22);
    float flashlight = pow(max(dot(-lightvector, lightdir),0),conewidth) * intensity * SpotFlashColor.w;
   dynamicContrib.xyz = dynamicContrib.xyz + (diffusemap.xyz*float3(flashlight,flashlight,flashlight));

   // paint
   float4 diffuseContrib = SurfColor * diffusemap * fLightOfShadow * GlobalSurfaceIntensity;
#ifdef PEROJECTLIGHT
    float4 specContrib = lighting.z * specmap * SurfColor * fLightOfShadow * GlobalSpecular;
#else
    float4 specContrib = lighting.z * specmap * SurfColor * fLightOfShadow * SpecularOverride * GlobalSpecular;
#endif
#ifdef SPECULARCAMERA
    float3 lightdirsc = normalize(float3(View._m02,View._m12,View._m22));
    lightdirsc = dot(-lightvector, lightdirsc);
    float4 lightingsc = lit(dot(lightvector,Nb),dot(lightvector,Nb),24);
    intensity = max(0, 1.5f - (viewspacePos.z/500.0f));
    lightingsc.z = lightingsc.z * intensity;
    specContrib = ( specContrib + ( (lightingsc.z *  SPECULARCAMERAINTENSITY ) * SurfColor * GlobalSpecular) ) * 0.5;
#endif

   // spherical ambience
   float fSkyFloorRatio = (1+dot(IN.WorldNormal.xyz,float3(0,1,0)))/2;
   float3 bouncelightcolor = lerp(FloorColor,SkyColor,fSkyFloorRatio) * diffusemap.xyz * 0.8;
   bouncelightcolor = bouncelightcolor + (diffusemap.xyz * 0.2);
    float4 ambContrib = (float4(bouncelightcolor,1) * ((AmbiColor * AmbiColorOverride)+float4(illummap.xxx,1))) * 2;
      
    // combine
#ifdef PEROJECTLIGHT
    float4 result = (diffuseContrib + ambContrib + specContrib + dynamicContrib) * SpecularOverride;
#else
    float4 result = diffuseContrib + ambContrib + specContrib + dynamicContrib;
#endif
   
   //calculate hud pixel-fog
    float4 cameraPos = mul(IN.WPos, View);
    float hudfogfactor = saturate((cameraPos.z- HudFogDist.x)/(HudFogDist.y - HudFogDist.x));
    float4 hudfogresult = lerp(result,float4(HudFogColor.xyz,0),hudfogfactor*HudFogColor.w);
   
   // original entity diffuse alpha with override
    hudfogresult.a = diffusemap.a * alphaoverride;    
   
   // entity effect control can slice alpha based on a world Y position
   float alphaslice = 1.0f - min(1,max(0,IN.WPos.y - EntityEffectControl.x)/50.0f);
   hudfogresult.a = hudfogresult.a * alphaslice;
   
   // final pixel color
   //return(rawlightmap);
   //return float4(fLightOfShadow.r,fLightOfShadow.g,fLightOfShadow.b,2.0) * 0.5;   

#ifdef DEBUGSHADOW
   if( iCurrentCascadeIndex == 0 ) {
      return float4( hudfogresult.r , hudfogresult.g*0.35 , hudfogresult.b*0.35 , hudfogresult.a );
   }
   if( iCurrentCascadeIndex == 1 ) {
      return float4( hudfogresult.r *0.35, hudfogresult.g , hudfogresult.b*0.35 , hudfogresult.a );
   }
   if( iCurrentCascadeIndex == 2 ) {
      return float4( hudfogresult.r*0.35 , hudfogresult.g*0.35 , hudfogresult.b , hudfogresult.a );
   }
   return hudfogresult;

#else
   return hudfogresult;   
#endif
}

float4 mainPS_medium(vertexOutput_low IN) : COLOR
{
   // clip
    clip(IN.clip);
   
   // lighting
    float3 V  = (eyePos - IN.WPos);  
    float3 Vn  = normalize(V); 
   
   // texture ref
    float4 diffusemap = tex2D(DiffuseSampler,IN.TexCoord.xy);

    // dynamic lighting
   float4 spotflashlighting = CalcSpotFlash (IN.WorldNormal,IN.WPos.xyz);   
   float4 dynamicContrib = CalcLighting (IN.WorldNormal,IN.WPos.xyz,Vn,diffusemap,float4(0,0,0,0)) + spotflashlighting;  
   
   // flash light system (flash light control carried in SpotFlashColor.w )
    float4 viewspacePos = mul(IN.WPos, View);
   float conewidth = 24;
    float intensity = max(0, 1.5f - (viewspacePos.z/500.0f));
    float3 lightvector = Vn;
    float3 lightdir = float3(View._m02,View._m12,View._m22);
    float flashlight = pow(max(dot(-lightvector, lightdir),0),conewidth) * intensity * SpotFlashColor.w;   
   dynamicContrib.xyz = dynamicContrib.xyz + (diffusemap.xyz*float3(flashlight,flashlight,flashlight));
   
   // choose between surface lighting and lightmap based on shadow strength
   float4 rawlightmap = tex2D(OccSampler,IN.TexCoord2.xy);

   // remove lightmapper blur artifacts.
   rawlightmap = clamp(rawlightmap,0.265,1.0);

   float4 fLightOfShadow = max(min(float4((rawlightmap.xyz-0.25f)*2,1),1),0); // 0.25=BLACK 0.75=WHITE -> 0-1 range
   
   // spherical ambience
   float fSkyFloorRatio = (1+dot(IN.WorldNormal.xyz,float3(0,1,0)))/2;
   float3 bouncelightcolor = lerp(FloorColor,SkyColor,fSkyFloorRatio) * diffusemap.xyz * 0.8;
   bouncelightcolor = bouncelightcolor + (diffusemap.xyz * 0.2);
   float4 ambContrib = float4(bouncelightcolor,1) * AmbiColor * AmbiColorOverride * 2;   
   float4 diffuseContrib = SurfColor * diffusemap * fLightOfShadow * GlobalSurfaceIntensity;

#ifdef PEROJECTLIGHT
   float4 result = (diffuseContrib + ambContrib + dynamicContrib) * SpecularOverride;
#else
   float4 result = diffuseContrib + ambContrib + dynamicContrib;
#endif

   //calculate hud pixel-fog
    float4 cameraPos = mul(IN.WPos, View);
    float hudfogfactor = saturate((cameraPos.z- HudFogDist.x)/(HudFogDist.y - HudFogDist.x));
    float4 hudfogresult = lerp(result,float4(HudFogColor.xyz,0),hudfogfactor*HudFogColor.w);
   
   // original entity diffuse alpha with override
    hudfogresult.a = diffusemap.a * alphaoverride;    
   
   // entity effect control can slice alpha based on a world Y position
   float alphaslice = 1.0f - min(1,max(0,IN.WPos.y - EntityEffectControl.x)/50.0f);
   hudfogresult.a = hudfogresult.a * alphaslice;

   // final pixel color
    return hudfogresult;   
}

float4 mainPS_lowest(vertexOutput_low IN) : COLOR
{
   // clip
    clip(IN.clip);
   
   // lighting
    float3 V  = (eyePos - IN.WPos);  
    float3 Vn  = normalize(V); 
   
   // CHEAPEST flash light system (flash light control carried in SpotFlashColor.w )
    float4 viewspacePos = mul(IN.WPos, View);
    float flashlight = (1.0f-min(1,viewspacePos.z/300.0f)) * SpotFlashColor.w;   
   
   // diffuse texture
    float4 diffusemap = tex2D(DiffuseSampler,IN.TexCoord.xy);

   // choose between surface lighting and lightmap based on shadow strength
   float4 rawlightmap = tex2D(OccSampler,IN.TexCoord2.xy);

   // remove lightmapper blur artifacts.
   rawlightmap = clamp(rawlightmap,0.265,1.0);

   float4 fLightOfShadow = max(min(float4((rawlightmap.xyz-0.25f)*2,1),1),0); // 0.25=BLACK 0.75=WHITE -> 0-1 range
      
   // spherical ambience
   float fSkyFloorRatio = (1+dot(IN.WorldNormal.xyz,float3(0,1,0)))/2;
   float3 bouncelightcolor = lerp(FloorColor,SkyColor,fSkyFloorRatio) * diffusemap.xyz * 0.8;
   bouncelightcolor = bouncelightcolor + (diffusemap.xyz * 0.2);
    float4 ambContrib = float4(bouncelightcolor,1) * AmbiColor * AmbiColorOverride * 2;   
   float4 diffuseContrib = SurfColor * diffusemap * fLightOfShadow * GlobalSurfaceIntensity;

#ifdef PEROJECTLIGHT
   float4 result = (diffuseContrib + ambContrib) * SpecularOverride;
#else
   float4 result = diffuseContrib + ambContrib;
#endif

   //calculate hud pixel-fog
    float4 cameraPos = mul(IN.WPos, View);
    float hudfogfactor = saturate((cameraPos.z- HudFogDist.x)/(HudFogDist.y - HudFogDist.x));
    float4 hudfogresult = lerp(result,float4(HudFogColor.xyz,0),hudfogfactor*HudFogColor.w);
   
   // original entity diffuse alpha with override
    hudfogresult.a = diffusemap.a * alphaoverride;    
   
   // entity effect control can slice alpha based on a world Y position
   float alphaslice = 1.0f - min(1,max(0,IN.WPos.y - EntityEffectControl.x)/50.0f);
   hudfogresult.a = hudfogresult.a * alphaslice;

   // final pixel color
    return hudfogresult;   
}

float4 mainPS_distant(vertexOutput_low IN) : COLOR
{
   // clip
    clip(IN.clip);
   
   // final pixel color
    return float4(1,1,1,1);   
}

float4 blackPS(vertexOutput_low IN) : COLOR
{
   clip(IN.clip); // all shaders should receive the clip value  
   if( tex2D(DiffuseSampler,IN.TexCoord.xy).a < TREEALPHACLIP ) 
   {
       clip(-1);
      return IN.vDepth;
   }
   return IN.vDepth;
}

/****** render depth information ***************/

vertexOutput_low mainVS_renderdepth(appdata IN)   
{
    vertexOutput_low OUT;
   float4 tempPos = mul(IN.Position, WorldViewProjection);
    OUT.Position = tempPos;
    OUT.LightVec = float3(0,0,0);
    float4 worldSpacePos = mul(IN.Position, World);
   OUT.vegshadowuv = float2(0,0);
    OUT.clip = dot(worldSpacePos, clipPlane);
    OUT.vDepth = 0;
    OUT.WorldNormal = float3(0,0,0);
    OUT.TexCoord2 = float2(0,0);
    OUT.TexCoord = IN.UV;
   worldSpacePos.w = 7000-mul( IN.Position, WorldView ).z;
    OUT.WPos = worldSpacePos;  
    OUT.vDepth = tempPos.z; 
    return OUT;
}
float4 mainPS_renderdepth(vertexOutput_low IN) : COLOR
{
    clip(IN.clip);
   clip(tex2D(DiffuseSampler,IN.TexCoord.xy).a-0.5);
   return IN.WPos.wxyz;
}

technique Highest
{
    pass RenderDepthPixelsPass
   <
      string RenderColorTarget = "[depthtexture]";
   >
    {
        // shaders
        VertexShader = compile vs_3_0 mainVS_renderdepth();
        PixelShader  = compile ps_3_0 mainPS_renderdepth();
        AlphaBlendEnable = false;
        alphafunc = greater;
      alpharef = 128;
        AlphaTestEnable = true;
    }
    pass MainPass
   <
      string RenderColorTarget = "";
   >
    {
        // shaders
        VertexShader = compile vs_3_0 mainVS_highest();
        PixelShader  = compile ps_3_0 mainPS_highest();
        alphafunc=greater;
        alpharef=128;
        AlphaBlendEnable = TRUE;
        AlphaTestEnable = true;
#ifdef DEBUGLEVEL
        FillMode = WIREFRAME;
#endif

    }
}

technique Medium
{
    pass RenderDepthPixelsPass
   <
      string RenderColorTarget = "[depthtexture]";
   >
    {
        // shaders
        VertexShader = compile vs_3_0 mainVS_renderdepth();
        PixelShader  = compile ps_3_0 mainPS_renderdepth();
        AlphaBlendEnable = false;
        alphafunc = greater;
      alpharef = 128;
        AlphaTestEnable = true;
    }
    pass MainPass
   <
      string RenderColorTarget = "";
   >
    {
        // shaders
        VertexShader = compile vs_3_0 mainVS_lowest();
        PixelShader  = compile ps_3_0 mainPS_medium();
        alphafunc=greater;
        alpharef=128;
        AlphaBlendEnable = TRUE;
        AlphaTestEnable = true;
    }
}

technique Lowest
{
    pass RenderDepthPixelsPass
   <
      string RenderColorTarget = "[depthtexture]";
   >
    {
        // shaders
        VertexShader = compile vs_3_0 mainVS_renderdepth();
        PixelShader  = compile ps_3_0 mainPS_renderdepth();
        AlphaBlendEnable = false;
        alphafunc = greater;
      alpharef = 128;
        AlphaTestEnable = true;
    }
    pass MainPass
   <
      string RenderColorTarget = "";
   >
    {
        // shaders
        VertexShader = compile vs_2_0 mainVS_lowest();
        PixelShader  = compile ps_2_0 mainPS_lowest();
        alphafunc=greater;
        alpharef=128;
        AlphaBlendEnable = TRUE;
        AlphaTestEnable = true;
    }
}

technique DepthMap
{
    pass p0
    {      
        VertexShader = compile vs_2_0 mainVS_lowest();
        PixelShader  = compile ps_2_0 blackPS();
        CullMode = CW; // NONE - CW - CCW
        alphafunc = greater;
      alpharef = 225;
        AlphaBlendEnable = FALSE;
        AlphaTestEnable = true;
   }
}

technique blacktextured
{
    pass P0
    {
        // shaders
        VertexShader = compile vs_2_0 mainVS_lowest();
        PixelShader  = compile ps_2_0 blackPS();
        CullMode = ccw;
        alphafunc=greater;
      alpharef=225 ;
        AlphaBlendEnable = FALSE;
        AlphaTestEnable = true;
    }
}
